use 5.010000;
use strict;
use warnings;
use Test::More;

use Test::Spelling;

add_stopwords(<DATA>);

all_pod_files_spelling_ok();


__DATA__
BackSpace
Ctrl
EOT
Kiem
Matthäus
PageDown
PageUp
ReadKey
SGR
SpaceBar
hjkl
lf
ll
stackoverflow
